@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,17 +19,13 @@
 #ifndef __DIR_POOL_H__
 #define __DIR_POOL_H__
 
+#include "gnome-cmd-dir-types.h"
 
-GnomeCmdDir *
-dir_pool_get (const gchar *dir_uri_str);
-
-void
-dir_pool_add (GnomeCmdDir *dir);
-
-void
-dir_pool_remove (GnomeCmdDir *dir);
-
-gboolean
-dir_pool_exists (GnomeCmdDir *dir);
+void         dir_pool_init (void);
+GnomeCmdDir *dir_pool_get (const gchar *dir_uri_str);
+void         dir_pool_add (GnomeCmdDir *dir);
+void         dir_pool_remove (GnomeCmdDir *dir);
+gboolean     dir_pool_exists (GnomeCmdDir *dir);
+void         dir_pool_show_state (void);
 
 #endif //__DIR_POOL_H__
