@@ -25,26 +25,20 @@
  */
 
 /*
- * Modified by Marcus Bjurman <marbj499@student.liu.se> 2001
+ * Modified by Marcus Bjurman <marbj499@student.liu.se> 2001-2003
  * The orginal comments are left intact above
  */
 
-#include <config.h>
-#include <string.h>
-#include <gtk/gtkarrow.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkclist.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkwindow.h>
-#include <gdk/gdkkeysyms.h>
-#include <gtk/gtkframe.h>
+#include "gnome-cmd-includes.h"
 #include "gnome-cmd-combo.h"
 #include "gnome-cmd-style.h"
+#include "gnome-cmd-data.h"
+
+enum {
+  ITEM_SELECTED,
+  POPWIN_HIDDEN,
+  LAST_SIGNAL
+};
 
 const gchar *gnome_cmd_combo_string_key = "gnome-cmd-combo-string-value";
 
@@ -53,37 +47,17 @@ const gchar *gnome_cmd_combo_string_key = "gnome-cmd-combo-string-value";
 
 static GtkHBoxClass *parent_class = NULL;
 
+static guint combo_signals[LAST_SIGNAL] = { 0 };
+
+
+static void
+gnome_cmd_combo_item_selected (GnomeCmdCombo *combo, gpointer data);
 
 
 /*******************************
  * Utility functions
  *******************************/
 
-static gint
-focus_idle (GnomeCmdCombo *combo)
-{
-	if (combo)
-    {
-		GDK_THREADS_ENTER ();
-		gtk_widget_grab_focus (combo->entry);
-		GDK_THREADS_LEAVE ();
-    }
-	return FALSE;
-}
-
-/*
-static void
-item_destroy (GtkObject *object)
-{
-	//FIXME: Why isn't this used ?
-	gchar *key;
-
-	key = gtk_object_get_data (object, gnome_cmd_combo_string_key);
-	if (key)
-		g_free (key);
-}
-*/
-
 static void
 size_allocate (GtkWidget     *widget,
 			   GtkAllocation *allocation)
@@ -189,7 +163,10 @@ get_pos (GnomeCmdCombo *combo, gint *x, gint *y, gint *height, gint *width)
 		}
     } while (old_width != alloc_width || old_height != work_height);
   
-	*width = widget->allocation.width + 100;
+	*width = widget->allocation.width;
+	if (*width < 200)
+		*width = 200;
+	
 	if (show_vscroll)
 		*height = avail_height;
 	else
@@ -201,32 +178,24 @@ get_pos (GnomeCmdCombo *combo, gint *x, gint *y, gint *height, gint *width)
 
 
 void
-popup_list (GnomeCmdCombo *combo, GtkWidget *file_list)
+gnome_cmd_combo_popup_list (GnomeCmdCombo *combo)
 {
 	gint height, width, x, y;
-	gint old_width, old_height;
-
-	combo->file_list = file_list;
-
-	old_width = combo->popwin->allocation.width;
-	old_height  = combo->popwin->allocation.height;
 
 	get_pos (combo, &x, &y, &height, &width);
 
-	/* workaround for gtk_scrolled_window_size_allocate bug */
-	if (old_width != width || old_height != height)
-    {
-		gtk_widget_hide (GTK_SCROLLED_WINDOW (combo->popup)->hscrollbar);
-		gtk_widget_hide (GTK_SCROLLED_WINDOW (combo->popup)->vscrollbar);
-    }
-
 	gtk_widget_set_uposition (combo->popwin, x, y);
 	gtk_widget_set_usize (combo->popwin, width, height);
 	gtk_widget_realize (combo->popwin);
 	gdk_window_resize (combo->popwin->window, width, height);
 	gtk_widget_show (combo->popwin);
 
-	gtk_widget_grab_focus (combo->popwin);
+	if (combo->first_popup) {
+		gtk_clist_unselect_all (GTK_CLIST (combo->list));
+		GTK_CLIST (combo->list)->focus_row = 0;
+		gtk_clist_select_row (GTK_CLIST (combo->list), 0, 0);
+		combo->first_popup = FALSE;
+	}
 }
 
 
@@ -237,177 +206,59 @@ popup_list (GnomeCmdCombo *combo, GtkWidget *file_list)
  * Callbacks
  *******************************/
 
-static int
-on_entry_key_press (GtkEntry *entry, GdkEventKey *event, GnomeCmdCombo *combo)
-{
-	return FALSE;
-}
-
-
-static gint
-on_entry_focus_out (GtkEntry *entry, GdkEventFocus *event, GnomeCmdCombo *combo)
-{
-
-	if (combo->value_in_list)
-    {
-		if (combo->ok_if_empty && !strcmp (gtk_entry_get_text (entry), ""))
-			return FALSE;
-
-#ifdef TEST
-		printf ("INVALID ENTRY: `%s'\n", gtk_entry_get_text (entry));
-#endif
-		gtk_grab_add (GTK_WIDGET (combo));
-		/* this is needed because if we call gtk_widget_grab_focus() 
-		   it isn't guaranteed it's the *last* call before the main-loop,
-		   so the focus can be lost anyway...
-		   the signal_emit_stop doesn't seem to work either...
-		*/
-		gtk_idle_add ((GtkFunction) focus_idle, combo);
-		
-		return TRUE;
-    }
-	return FALSE;
-}
-
-
-static void        
-on_entry_activate (GtkWidget        *widget,
-				   GnomeCmdCombo    *combo)
-{
-	popup_list (combo, NULL);
-
-	if (!GTK_WIDGET_HAS_FOCUS (combo->entry))
-		gtk_widget_grab_focus (combo->entry);
-
-	gtk_grab_add (combo->popwin);
-
-	gdk_pointer_grab (combo->popwin->window, TRUE,
-					  GDK_BUTTON_PRESS_MASK | 
-					  GDK_BUTTON_RELEASE_MASK |
-					  GDK_POINTER_MOTION_MASK, 
-					  NULL, NULL, GDK_CURRENT_TIME);
-}
-
-
-static void
-on_entry_changed (GtkEntry *entry, GnomeCmdCombo *combo)
-{
-	gtk_grab_remove (GTK_WIDGET (combo));
-}
-
-
-
-
 static gboolean        
 on_popup_button_press (GtkWidget        *button,
 					   GdkEventButton   *event,
 					   GnomeCmdCombo    *combo)
-{
-	if (!GTK_WIDGET_HAS_FOCUS (combo->entry))
-		gtk_widget_grab_focus (combo->entry);
-	if (!combo->current_button && (event->button == 1))
-		popup_list (combo, NULL);
-
-	combo->current_button = event->button;
-  
-	//GTK_LIST (combo->list)->drag_selection = TRUE;
-	
-	gdk_pointer_grab (combo->list->window, TRUE,
-					  GDK_POINTER_MOTION_HINT_MASK |
-					  GDK_BUTTON1_MOTION_MASK |
-					  GDK_BUTTON_RELEASE_MASK,
-					  NULL, NULL, event->time);
-		
-	gtk_grab_add (combo->list);
-
-	return FALSE;
-}
-
-
-static void         
-on_popup_button_leave_notify (GtkWidget        *button,
-							  GdkEventCrossing *event,
-							  GnomeCmdCombo    *combo)
-{
-	if (combo->current_button)
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (button), "leave_notify_event");
-}
-
-
-
-
-
-static void
-on_list_select_row (GtkCList *list, gint row, gint column,
-					GdkEventButton *event, GnomeCmdCombo *combo)
-{
-	gchar *text;
-
-	gtk_clist_get_text (list, row, 0, &text);
-	gtk_entry_set_text (GTK_ENTRY (combo->entry), text);
-}
-
-
-static void
-on_list_unselect_row (GtkCList *list, gint row, gint column,
-					  GdkEventButton *event, GnomeCmdCombo *combo)
-{
-}
-
-
-static gboolean
-on_list_button_release (GtkWidget *widget, GdkEvent *event, GnomeCmdCombo *combo)
 {
 	GtkWidget *child;
 
-	if ((combo->current_button != 0) && (event->button.button == 1))
-    {
-		/* This was the initial button press */
-		GdkEventCrossing tmp_event;
+	if (!event) return FALSE;
+	if (event->button != 1) return FALSE;
 
-		combo->current_button = 0;
+	// Check to see if we released inside the button
+	child = gtk_get_event_widget ((GdkEvent*) event);
+	
+	while (child && child != (combo->button))
+		child = child->parent;
+	
+	if (child == combo->button) {
+		return FALSE;
+	}
 
-		if (widget != combo->button)
-			gtk_widget_event (combo->button, event);
+//	gtk_widget_hide (combo->popwin);
+//	combo->is_popped = FALSE;
+	return TRUE;
+}
 
-		/* Un-pre-hightlight */      
-		tmp_event.type = GDK_LEAVE_NOTIFY;
-		tmp_event.window = combo->button->window;
-		tmp_event.send_event = TRUE;
-		tmp_event.subwindow = NULL;
-		tmp_event.detail = GDK_NOTIFY_ANCESTOR;
-      
-		gtk_widget_event (combo->button, (GdkEvent *)&tmp_event);
 
-		/* Check to see if we released inside the button */
-		child = gtk_get_event_widget ((GdkEvent*) event);
+static gboolean        
+on_popup_button_release (GtkWidget        *button,
+						 GdkEventButton   *event,
+						 GnomeCmdCombo    *combo)
+{
+	GtkWidget *child;
+	
+	if (!event) return FALSE;
+	if (event->button != 1) return FALSE;
 
-		while (child && child != (combo->button))
-			child = child->parent;
-
-		if (child == combo->button)
-		{
-			gtk_grab_add (combo->popwin);
-			gdk_pointer_grab (combo->popwin->window, TRUE,
-							  GDK_BUTTON_PRESS_MASK | 
-							  GDK_BUTTON_RELEASE_MASK |
-							  GDK_POINTER_MOTION_MASK, 
-							  NULL, NULL, GDK_CURRENT_TIME);
-
-			return FALSE;
+	// Check to see if we released inside the button
+	child = gtk_get_event_widget ((GdkEvent*) event);
+	
+	while (child && child != (combo->button))
+		child = child->parent;
+	
+	if (child == combo->button) {
+		if (combo->is_popped) {
+			gtk_widget_hide (combo->popwin);
+			combo->is_popped = FALSE;
 		}
-    }
-	else
-    {
-		/* The user has clicked inside the popwin and released */
-		if (GTK_WIDGET_HAS_GRAB (combo->popwin))
-		{
-			gtk_grab_remove (combo->popwin);
-			gdk_pointer_ungrab (event->button.time);
+		else {
+			gnome_cmd_combo_popup_list (combo);
+			combo->is_popped = TRUE;
 		}
-    }
-  
-	gtk_widget_hide (combo->popwin);
+		return FALSE;
+	}
 
 	return TRUE;
 }
@@ -416,35 +267,54 @@ on_list_button_release (GtkWidget *widget, GdkEvent *event, GnomeCmdCombo *combo
 static int
 on_list_key_press (GtkWidget *widget, GdkEventKey *event, GnomeCmdCombo *combo)
 {
-	if (event->keyval == GDK_Escape)
-    {
-		if (GTK_WIDGET_HAS_GRAB (combo->popwin))
-		{
-			gtk_grab_remove (combo->popwin);
-			gdk_pointer_ungrab (GDK_CURRENT_TIME);
-		}
-		
+	if (event->keyval == GDK_Escape) {
 		gtk_widget_hide (combo->popwin);
-		
-		if (GTK_WIDGET_HAS_GRAB (combo->button))
-		{
-			combo->current_button = 0;
-			GTK_BUTTON (combo->button)->in_button = FALSE;
-			gtk_button_released (GTK_BUTTON (combo->button));
-			gtk_grab_remove (combo->button);
-		}
+		combo->is_popped = FALSE;
+		return TRUE;
+    }
 
-		if (GTK_WIDGET_HAS_GRAB (combo->list))
-		{
-			gtk_grab_remove (combo->list);
+	return FALSE;
+}
 
-		}
-		
-		if (combo->file_list)
-		{
-			gtk_widget_grab_focus (combo->file_list);
-			combo->file_list = NULL;
-		}
+
+static gboolean        
+on_popwin_button_released (GtkWidget        *button,
+						  GdkEventButton   *event,
+						  GnomeCmdCombo    *combo)
+{
+	GtkWidget *child;
+	
+	if (!event) return FALSE;
+	if (event->button != 1) return FALSE;
+
+	// Check to see if we clicked inside the popwin
+	child = gtk_get_event_widget ((GdkEvent*) event);	
+	while (child && child != (combo->popwin))
+		child = child->parent;
+	
+	if (child != combo->popwin) {
+		// We clicked outside the popwin
+		gtk_widget_hide (combo->popwin);
+		combo->is_popped = FALSE;
+		return TRUE;
+	}
+	
+	return FALSE;
+}
+
+
+static int
+on_popwin_keypress (GtkWidget *widget, GdkEventKey *event, GnomeCmdCombo *combo)
+{
+	if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter) {
+		gpointer data;
+		gint row;
+
+		row = GTK_CLIST (combo->list)->focus_row;
+		if (row < 0) return TRUE;
+
+		data = gtk_clist_get_row_data (GTK_CLIST (combo->list), row);
+		gtk_signal_emit (GTK_OBJECT (combo), combo_signals[ITEM_SELECTED], data);
 		
 		return TRUE;
     }
@@ -454,133 +324,38 @@ on_list_key_press (GtkWidget *widget, GdkEventKey *event, GnomeCmdCombo *combo)
 
 
 static void
-on_list_scroll_vertical                  (GtkCList          *clist,
-										  GtkScrollType      scroll_type,
-										  gfloat             position,
-										  GnomeCmdCombo     *combo)
+on_list_select_row (GtkCList *list, gint row, gint col,
+					GdkEventButton *event, GnomeCmdCombo *combo)
 {
-	if (clist->focus_row >= 0)
-		gtk_clist_select_row (clist, clist->focus_row, 0);
+	GdkEventKey event2;
+	if (!event) return;
+
+	event2.type = GDK_KEY_PRESS;
+	event2.keyval = GDK_Return;
+	event2.state = 0;
+
+	on_popwin_keypress (GTK_WIDGET (combo->popwin), &event2, combo);
 }
 
 
-static gboolean
-on_popupwin_button_press (GtkWidget *widget, GdkEvent *event, GnomeCmdCombo *combo)
+static void
+on_popwin_show (GtkWidget *widget,
+				GnomeCmdCombo *combo)
 {
-	GtkWidget *child;
+	gtk_grab_add (combo->popwin);
+	gtk_widget_grab_focus (combo->list);
+}
 
-	child = gtk_get_event_widget (event);
 
-	/* We don't ask for button press events on the grab widget, so
-	 *  if an event is reported directly to the grab widget, it must
-	 *  be on a window outside the application (and thus we remove
-	 *  the popup window). Otherwise, we check if the widget is a child
-	 *  of the grab widget, and only remove the popup window if it
-	 *  is not.
-	 */
-	
-	if (child != widget)
-    {
-		while (child)
-		{
-			if (child == widget)
-				return FALSE;
-			child = child->parent;
-		}
-    }
-	
-	gtk_widget_hide (combo->popwin);
+static void
+on_popwin_hide (GtkWidget *widget,
+				GnomeCmdCombo *combo)
+{
 	gtk_grab_remove (combo->popwin);
-	gdk_pointer_ungrab (event->button.time);
-	gtk_grab_remove (combo->list);
-	
-	return TRUE;
+	gtk_signal_emit (GTK_OBJECT (combo), combo_signals[POPWIN_HIDDEN]);
 }
 
 
-static int
-on_popupwin_key_press (GtkWidget *widget, GdkEventKey *event, GnomeCmdCombo *combo)
-{
-	if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter)
-    {
-		if (GTK_WIDGET_VISIBLE (combo->popwin))
-		{
-			gtk_widget_hide (combo->popwin);
-	  
-			if (GTK_WIDGET_HAS_GRAB (combo->popwin))
-			{
-				gtk_grab_remove (combo->popwin);
-				gdk_pointer_ungrab (event->time);
-			}
-		}
-
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
-
-		if (GTK_WIDGET_HAS_GRAB (combo->list))
-		{
-			gtk_grab_remove (combo->list);
-
-		}
-		
-		if (combo->file_list)
-		{
-			gtk_widget_grab_focus (combo->file_list);
-			combo->file_list = NULL;
-		}
-
-		return TRUE;
-    }
-
-	return FALSE;
-}
-
-
-
-
-static gint         
-on_button_enter_notify (GtkWidget        *widget,
-						GdkEventCrossing *event,
-						GnomeCmdCombo    *combo)
-{
-	GtkWidget *event_widget;
-
-	event_widget = gtk_get_event_widget ((GdkEvent*) event);
-  
-	if ((event_widget == combo->list) &&
-		(combo->current_button != 0) && 
-		(!GTK_WIDGET_HAS_GRAB (combo->list)))
-    {
-		GdkEvent tmp_event;
-		gint x, y;
-		GdkModifierType mask;
-
-		gtk_grab_remove (combo->popwin);
-
-		/* Transfer the grab over to the list by synthesizing
-		 * a button press event
-		 */
-		gdk_window_get_pointer (combo->list->window, &x, &y, &mask);
-
-		tmp_event.button.type = GDK_BUTTON_PRESS;
-		tmp_event.button.window = combo->list->window;
-		tmp_event.button.send_event = TRUE;
-		tmp_event.button.time = GDK_CURRENT_TIME; /* bad */
-		tmp_event.button.x = x;
-		tmp_event.button.y = y;
-		/* We leave all the XInput fields unfilled here, in the expectation
-		 * that GtkList doesn't care.
-		 */
-		tmp_event.button.button = combo->current_button;
-		tmp_event.button.state = mask;
-
-		gtk_widget_event (combo->list, &tmp_event);
-    }
-
-	return FALSE;
-}
-
-
-
 /*******************************
  * Gtk class implementation
  *******************************/
@@ -596,47 +371,69 @@ destroy (GtkObject *combo)
 static void
 class_init (GnomeCmdComboClass *klass)
 {
-	GtkObjectClass *oclass;
+	GtkObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 
 	parent_class = gtk_type_class (gtk_hbox_get_type ());
-	oclass = (GtkObjectClass *) klass;
+	object_class = (GtkObjectClass *) klass;
 	widget_class = (GtkWidgetClass *) klass;
 
-	oclass->destroy = destroy;
-  
+	combo_signals[ITEM_SELECTED] =
+		gtk_signal_new ("item_selected",
+			GTK_RUN_LAST,
+		    object_class->type,
+		    GTK_SIGNAL_OFFSET (GnomeCmdComboClass, item_selected),
+		    gtk_marshal_NONE__POINTER,
+		    GTK_TYPE_NONE,
+			1, GTK_TYPE_POINTER);
+
+	combo_signals[POPWIN_HIDDEN] =
+		gtk_signal_new ("popwin_hidden",
+			GTK_RUN_LAST,
+		    object_class->type,
+		    GTK_SIGNAL_OFFSET (GnomeCmdComboClass, popwin_hidden),
+		    gtk_marshal_NONE__NONE,
+		    GTK_TYPE_NONE,
+			0);
+	
+	gtk_object_class_add_signals (object_class, combo_signals, LAST_SIGNAL);
+	
+	object_class->destroy = destroy;  
 	widget_class->size_allocate = size_allocate;
+	klass->item_selected = gnome_cmd_combo_item_selected;
+	klass->popwin_hidden = NULL;
 }
 
 static void
 init (GnomeCmdCombo *combo)
 {
-	int i;
 	GtkWidget *arrow;
 	GtkWidget *frame;
 	GtkWidget *event_box;
 	GdkCursor *cursor;
-	gchar *list_titles[] = {"name", "comment", NULL};
 
 	combo->value_in_list = 0;
 	combo->ok_if_empty = 1;
-	combo->file_list = NULL;
+	combo->highest_pixmap = 20;
+	combo->widest_pixmap = 20;
+	combo->is_popped = FALSE;
 	
 	combo->entry = gtk_entry_new ();
 	gtk_widget_ref (combo->entry);
 	gtk_object_set_data_full (GTK_OBJECT (combo),
 							  "entry", combo->entry,
 							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_set_style (combo->entry, list_style);
+//	gtk_widget_set_style (combo->entry, list_style);
 	gtk_widget_show (combo->entry);
+	gtk_widget_set_usize (combo->entry, 60, -1);
 	
 	combo->button = gtk_button_new ();
 	gtk_widget_ref (combo->button);
+	gtk_button_set_relief (GTK_BUTTON (combo->button), gnome_cmd_data_get_button_relief ());
 	gtk_object_set_data_full (GTK_OBJECT (combo),
 							  "button", combo->button,
 							  (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (combo->button);
-	combo->current_button = 0;
 	
 	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
 	gtk_widget_ref (arrow);
@@ -650,28 +447,13 @@ init (GnomeCmdCombo *combo)
 	gtk_box_pack_end (GTK_BOX (combo), combo->button, FALSE, FALSE, 0);
 	GTK_WIDGET_UNSET_FLAGS (combo->button, GTK_CAN_FOCUS);
 
-	/* connect entry signals */
-	gtk_signal_connect (
-		GTK_OBJECT (combo->entry), "changed",
-		(GtkSignalFunc) on_entry_changed, combo);
-	gtk_signal_connect (
-		GTK_OBJECT (combo->entry), "key-press-event",
-		(GtkSignalFunc) on_entry_key_press, combo);
-	gtk_signal_connect_after (
-		GTK_OBJECT (combo->entry), "focus-out-event",
-		(GtkSignalFunc) on_entry_focus_out, combo);
-	gtk_signal_connect (
-		GTK_OBJECT (combo->entry), "activate",
-		(GtkSignalFunc) on_entry_activate, combo);
-
 	/* connect button signals */
+	gtk_signal_connect_after (
+		GTK_OBJECT (combo->button), "button-release-event",
+		(GtkSignalFunc) on_popup_button_release, combo);
 	gtk_signal_connect_after (
 		GTK_OBJECT (combo->button), "button-press-event",
 		(GtkSignalFunc) on_popup_button_press, combo);
-	gtk_signal_connect (
-		GTK_OBJECT (combo->button), "leave-notify-event",
-		(GtkSignalFunc) on_popup_button_leave_notify, combo);
-
 	
 	combo->popwin = gtk_window_new (GTK_WINDOW_POPUP);
 	gtk_widget_ref (combo->popwin);
@@ -680,16 +462,23 @@ init (GnomeCmdCombo *combo)
 							  (GtkDestroyNotify) gtk_widget_unref);	
 	gtk_window_set_policy (GTK_WINDOW (combo->popwin), 1, 1, 0);
 
+	gtk_widget_set_events (combo->popwin, GDK_KEY_PRESS_MASK);
+	gtk_widget_set_events (combo->popwin, GDK_BUTTON_PRESS_MASK);
+
 	/* connect popupwin signals */
 	gtk_signal_connect (
-		GTK_OBJECT (combo->popwin), "key-press-event",
-		GTK_SIGNAL_FUNC (on_popupwin_key_press), combo);
+		GTK_OBJECT (combo->popwin), "button-release-event",
+		GTK_SIGNAL_FUNC (on_popwin_button_released), combo);
 	gtk_signal_connect (
-		GTK_OBJECT (combo->popwin), "button-press-event",
-		GTK_SIGNAL_FUNC (on_popupwin_button_press), combo);
+		GTK_OBJECT (combo->popwin), "key-press-event",
+		GTK_SIGNAL_FUNC (on_popwin_keypress), combo);
+	gtk_signal_connect (
+		GTK_OBJECT (combo->popwin), "show",
+		GTK_SIGNAL_FUNC (on_popwin_show), combo);
+	gtk_signal_connect (
+		GTK_OBJECT (combo->popwin), "hide",
+		GTK_SIGNAL_FUNC (on_popwin_hide), combo);
   
-	gtk_widget_set_events (combo->popwin, GDK_KEY_PRESS_MASK);
-
 	event_box = gtk_event_box_new ();
 	gtk_widget_ref (event_box);
 	gtk_object_set_data_full (GTK_OBJECT (combo),
@@ -697,8 +486,8 @@ init (GnomeCmdCombo *combo)
 							  (GtkDestroyNotify) gtk_widget_unref);	
 	gtk_container_add (GTK_CONTAINER (combo->popwin), event_box);
 	gtk_widget_show (event_box);
-
 	gtk_widget_realize (event_box);
+	
 	cursor = gdk_cursor_new (GDK_TOP_LEFT_ARROW);
 	gdk_window_set_cursor (event_box->window, cursor);
 	gdk_cursor_destroy (cursor);
@@ -723,61 +512,6 @@ init (GnomeCmdCombo *combo)
 	GTK_WIDGET_UNSET_FLAGS (GTK_SCROLLED_WINDOW (combo->popup)->vscrollbar, GTK_CAN_FOCUS);
 	gtk_container_add (GTK_CONTAINER (frame), combo->popup);
 	gtk_widget_show (combo->popup);
-
-	combo->list = gtk_clist_new_with_titles (2,list_titles);
-	gtk_widget_ref (combo->list);
-	gtk_object_set_data_full (GTK_OBJECT (combo),
-							  "combo->list", combo->list,
-							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_clist_set_selection_mode (GTK_CLIST (combo->list), GTK_SELECTION_SINGLE);
-	gtk_widget_set_style (combo->list, list_style);
-	for ( i=0 ; i<GTK_CLIST (combo->list)->columns ; i++ )
-		gtk_widget_set_style (
-			GTK_BIN(
-				GTK_BIN (
-					GTK_CLIST (combo->list)->column[i].button)->child)->child,
-				main_style);
-	
-	/* We'll use enter notify events to figure out when to transfer
-	 * the grab to the list
-	 */
-	gtk_widget_set_events (combo->list, GDK_ENTER_NOTIFY_MASK);
-
-	gtk_scrolled_window_add_with_viewport (
-		GTK_SCROLLED_WINDOW (combo->popup), combo->list);
-	gtk_container_set_focus_vadjustment (
-		GTK_CONTAINER (combo->list),
-		gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (combo->popup)));
-	gtk_container_set_focus_hadjustment (
-		GTK_CONTAINER (combo->list),
-		gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (combo->popup)));
-	gtk_widget_show (combo->list);
-
-	/* connect list signals */
-	gtk_signal_connect (
-		GTK_OBJECT (combo->list), "select-row",
-		GTK_SIGNAL_FUNC (on_list_select_row), combo);
-	gtk_signal_connect (
-		GTK_OBJECT (combo->list), "unselect-row",
-		GTK_SIGNAL_FUNC (on_list_unselect_row), combo);
-	gtk_signal_connect_after (
-		GTK_OBJECT (combo->list), "button-release-event",
-		GTK_SIGNAL_FUNC (on_list_button_release), combo);
-	gtk_signal_connect (
-		GTK_OBJECT (combo->list), "key-press-event",
-		GTK_SIGNAL_FUNC (on_list_key_press), combo);
-	gtk_signal_connect_after (
-		GTK_OBJECT (combo->list), "scroll-vertical",
-		GTK_SIGNAL_FUNC (on_list_scroll_vertical), combo);
-
-	
-	/* We connect here on the button, because we'll have a grab on it
-	 * when the event occurs. But we are actually interested in enters
-	 * for the combo->list.
-	 */
-	gtk_signal_connect (
-		GTK_OBJECT (combo->button), "enter-notify-event",
-		GTK_SIGNAL_FUNC (on_button_enter_notify), combo);
 }
 
 
@@ -810,32 +544,37 @@ gnome_cmd_combo_get_type (void)
 }
 
 GtkWidget *
-gnome_cmd_combo_new (void)
+gnome_cmd_combo_new (gint num_cols, gchar **col_titles)
 {
-	return GTK_WIDGET (gtk_type_new (gnome_cmd_combo_get_type ()));
-}
+	GnomeCmdCombo *combo =  gtk_type_new (gnome_cmd_combo_get_type ());
 
+	if (col_titles)
+		combo->list = gnome_cmd_clist_new_with_titles (num_cols, col_titles);
+	else
+		combo->list = gnome_cmd_clist_new (num_cols);
 
-void
-gnome_cmd_combo_set_popdown_strings (GnomeCmdCombo *combo, GList *strings)
-{
-	GList *list = strings;
+	combo->first_popup = TRUE;
+	
+	gtk_widget_ref (combo->list);
+	gtk_object_set_data_full (GTK_OBJECT (combo),
+							  "combo->list", combo->list,
+							  (GtkDestroyNotify) gtk_widget_unref);
 
-	g_return_if_fail (combo != NULL);
-	g_return_if_fail (GNOME_CMD_IS_COMBO (combo));
-	g_return_if_fail (strings != NULL);
+	/* We'll use enter notify events to figure out when to transfer
+	 * the grab to the list
+	 */
+	gtk_container_add (GTK_CONTAINER (combo->popup), combo->list);
+	gtk_widget_show (combo->list);
 
-	gtk_clist_clear (GTK_CLIST (combo->list));
+	/* connect list signals */
+	gtk_signal_connect_after (
+		GTK_OBJECT (combo->list), "select-row",
+		GTK_SIGNAL_FUNC (on_list_select_row), combo);
+	gtk_signal_connect (
+		GTK_OBJECT (combo->list), "key-press-event",
+		GTK_SIGNAL_FUNC (on_list_key_press), combo);
 
-	while (list)
-    {
-		gchar *text[3];
-		text[0] = (gchar*)list->data;
-		text[1] = "description";
-		text[2] = NULL;
-		gtk_clist_append (GTK_CLIST (combo->list), text);
-		list = list->next;
-    }
+	return GTK_WIDGET (combo);
 }
 
 
@@ -846,14 +585,52 @@ gnome_cmd_combo_clear (GnomeCmdCombo *combo)
 }
 
 
-void
-gnome_cmd_combo_append (GnomeCmdCombo *combo, gchar **text)
+gint
+gnome_cmd_combo_append (GnomeCmdCombo *combo, gchar **text, gpointer data)
 {
-	g_return_if_fail (combo != NULL);
-	g_return_if_fail (GNOME_CMD_IS_COMBO (combo));
-	g_return_if_fail (text != NULL);
+	gint row;
+	
+	g_return_val_if_fail (GNOME_CMD_IS_COMBO (combo), -1);
+	g_return_val_if_fail (text != NULL, -1);
 
-	gtk_clist_append (GTK_CLIST (combo->list), text);
+	row = gtk_clist_append (GTK_CLIST (combo->list), text);
+	gtk_clist_set_row_data (GTK_CLIST (combo->list), row, data);
+	return row;
+}
+
+
+gint
+gnome_cmd_combo_insert (GnomeCmdCombo *combo, gchar **text, gpointer data)
+{
+	gint row;
+	
+	g_return_val_if_fail (GNOME_CMD_IS_COMBO (combo), -1);
+	g_return_val_if_fail (text != NULL, -1);
+
+	row = gtk_clist_insert (GTK_CLIST (combo->list), 0, text);
+	gtk_clist_set_row_data (GTK_CLIST (combo->list), row, data);
+	return row;
+}
+
+
+void
+gnome_cmd_combo_set_pixmap (GnomeCmdCombo *combo,
+							gint           row,
+							gint           col,
+							GnomeCmdPixmap *pixmap)
+{
+	g_return_if_fail (GNOME_CMD_IS_COMBO (combo));
+	g_return_if_fail (pixmap != NULL);
+	
+	gtk_clist_set_pixmap (GTK_CLIST (combo->list), row, col, pixmap->pixmap, pixmap->mask);
+	if (pixmap->height > combo->highest_pixmap) {
+		gtk_clist_set_row_height (GTK_CLIST (combo->list), pixmap->height);
+		combo->highest_pixmap = pixmap->height;
+	}
+	if (pixmap->width > combo->widest_pixmap) {
+		gtk_clist_set_column_width (GTK_CLIST (combo->list), 0, pixmap->width);
+		combo->widest_pixmap = pixmap->width;
+	}
 }
 
 
@@ -863,7 +640,7 @@ gnome_cmd_combo_set_selection (GnomeCmdCombo *combo, const gchar *text)
 	g_return_if_fail (GNOME_CMD_IS_COMBO (combo));
 	g_return_if_fail (text != NULL);
 
-	gtk_entry_set_text (GTK_ENTRY (combo->entry), text);	
+	gtk_entry_set_text (GTK_ENTRY (combo->entry), text);
 }
 
 
@@ -873,12 +650,31 @@ gnome_cmd_combo_update_style (GnomeCmdCombo *combo)
 	gint i;
 	
 	gtk_widget_set_style (combo->entry, list_style);
-	gtk_widget_set_style (combo->list, list_style);
-	
-	for ( i=0 ; i<GTK_CLIST (combo->list)->columns ; i++ )
-		gtk_widget_set_style (
-			GTK_BIN(
-				GTK_BIN (
-					GTK_CLIST (combo->list)->column[i].button)->child)->child,
-				main_style);	
+	gnome_cmd_clist_update_style (GNOME_CMD_CLIST (combo->list));
+
+	for ( i=0 ; i<GTK_CLIST (combo->list)->columns ; i++ ) {
+		GtkWidget *button;
+		GtkWidget *child;
+
+		button = GTK_CLIST (combo->list)->column[i].button;
+		if (!button) return;
+
+		child = GTK_BIN (button)->child;
+		if (!child) return;
+
+		child = GTK_BIN (child)->child;
+		if (!child) return;
+		
+		gtk_widget_modify_style (child, main_rc_style);
+	}
+}
+
+
+static void
+gnome_cmd_combo_item_selected (GnomeCmdCombo *combo, gpointer data)
+{
+	g_return_if_fail (GNOME_CMD_IS_COMBO (combo));
+
+	gtk_widget_hide (combo->popwin);
+	combo->is_popped = FALSE;
 }
