@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,13 @@
 typedef struct _GnomeCmdData GnomeCmdData;
 typedef struct _GnomeCmdDataPrivate GnomeCmdDataPrivate;
 
+#include "gnome-cmd-ftp-server-types.h"
+#include "gnome-cmd-device.h"
+#include "gnome-cmd-app.h"
+#include "filter.h"
+#include "gnome-cmd-types.h"
+#include "history.h"
+
 
 struct _GnomeCmdData
 {
@@ -33,25 +40,37 @@ typedef struct
 	GdkColor *sel_fg, *sel_bg;
 	GdkColor *norm_fg, *norm_bg;
 	GdkColor *curs_fg, *curs_bg;
-	GdkColor *term_fg, *term_bg;
 } GnomeCmdLayoutColors;
 
-typedef struct
-{
-	GdkColor *modified;
-	GdkColor *updated;
-	GdkColor *unknown;
-	GdkColor *added;
-	GdkColor *removed;
-	GdkColor *conflict;
-} GnomeCmdCvsUpdateColors;
-
 typedef enum
 {
 	RIGHT_BUTTON_POPUPS_MENU,
 	RIGHT_BUTTON_SELECTS
 } GnomeCmdRightMouseButtonMode;
 
+typedef struct
+{
+	GList *name_patterns;
+	GList *content_patterns;
+	GList *directories;
+	gboolean recursive;
+	gboolean case_sens;	
+} SearchDefaults;
+
+typedef struct
+{
+	GList *patterns;
+	History *templates;
+	guint counter_start;
+	guint counter_precision;
+	guint counter_increment;
+	gboolean auto_update;
+	gint width,height;
+	gint pat_col_widths;
+	gint res_col_widths;
+	gint sep_value;
+} AdvrenameDefaults;
+
 
 GnomeCmdData*
 gnome_cmd_data_new                       (void);
@@ -65,6 +84,9 @@ gnome_cmd_data_save                      (void);
 void
 gnome_cmd_data_load                      (void);
 
+void
+gnome_cmd_data_load_more                 (void);
+
 void
 gnome_cmd_data_add_ftp_server            (GnomeCmdFtpServer *server);
 
@@ -77,6 +99,33 @@ gnome_cmd_data_get_ftp_servers           (void);
 void
 gnome_cmd_data_set_ftp_servers           (GList *servers);
 
+
+void
+gnome_cmd_data_add_device                (GnomeCmdDevice *device);
+
+void
+gnome_cmd_data_remove_device             (GnomeCmdDevice *device);
+
+GList*
+gnome_cmd_data_get_devices               (void);
+
+void
+gnome_cmd_data_set_devices               (GList *devices);
+
+
+void
+gnome_cmd_data_add_fav_app                (GnomeCmdApp *app);
+
+void
+gnome_cmd_data_remove_fav_app             (GnomeCmdApp *app);
+
+GList*
+gnome_cmd_data_get_fav_apps               (void);
+
+void
+gnome_cmd_data_set_fav_apps               (GList *apps);
+
+
 const gchar *
 gnome_cmd_data_get_ftp_anonymous_password (void);
 
@@ -89,6 +138,12 @@ gnome_cmd_data_get_size_disp_mode        (void);
 void
 gnome_cmd_data_set_size_disp_mode        (GnomeCmdSizeDispMode mode);
 
+GnomeCmdPermDispMode
+gnome_cmd_data_get_perm_disp_mode        (void);
+
+void
+gnome_cmd_data_set_perm_disp_mode        (GnomeCmdPermDispMode mode);
+
 GnomeCmdDateFormat
 gnome_cmd_data_get_date_format           (void);
 
@@ -101,8 +156,11 @@ gnome_cmd_data_get_layout                (void);
 void
 gnome_cmd_data_set_layout                (GnomeCmdLayout layout);
 
-GnomeCmdCvsUpdateColors*
-gnome_cmd_data_get_cvs_update_colors    (void);
+gint
+gnome_cmd_data_get_list_row_height       (void);
+
+void
+gnome_cmd_data_set_list_row_height       (gint height);
 
 GnomeCmdLayoutColors*
 gnome_cmd_data_get_layout_colors    (void);
@@ -120,17 +178,6 @@ gnome_cmd_data_set_hidden_filter        (gboolean value);
 gboolean
 gnome_cmd_data_get_hidden_filter        (void);
 
-/*
-void
-gnome_cmd_data_add_cvs_server           (GnomeCmdCvsServer *cvs_server);
-
-void
-gnome_cmd_data_remove_cvs_server        (GnomeCmdCvsServer *cvs_server);
-
-GList*
-gnome_cmd_data_get_cvs_servers          (void);
-*/
-
 void
 gnome_cmd_data_set_position             (gint width, gint height);
 
@@ -149,9 +196,6 @@ gnome_cmd_data_set_editor               (const gchar *command);
 void
 gnome_cmd_data_set_differ               (const gchar *command);
 
-void
-gnome_cmd_data_set_cvs_differ           (const gchar *command);
-
 const gchar *
 gnome_cmd_data_get_viewer               (void); 
 
@@ -161,9 +205,6 @@ gnome_cmd_data_get_editor               (void);
 const gchar *
 gnome_cmd_data_get_differ               (void); 
 
-const gchar *
-gnome_cmd_data_get_cvs_differ           (void); 
-
 gboolean
 gnome_cmd_data_get_case_sens_sort       (void); 
 
@@ -171,22 +212,16 @@ void
 gnome_cmd_data_set_case_sens_sort       (gboolean value);
 
 const gchar *
-gnome_cmd_data_get_font                 (void);
+gnome_cmd_data_get_list_font             (void);
+
+const gchar *
+gnome_cmd_data_get_misc_font             (void);
 
 void
-gnome_cmd_data_set_font                 (const gchar *font);
+gnome_cmd_data_set_list_font             (const gchar *list_font);
 
 void
-gnome_cmd_data_set_cvsroot              (const gchar *cvsroot);
-
-void
-gnome_cmd_data_set_cvs_over_ssh         (gboolean value);
-
-const gchar*
-gnome_cmd_data_get_cvsroot              (void);
-
-gboolean
-gnome_cmd_data_get_cvs_over_ssh         (void);
+gnome_cmd_data_set_misc_font             (const gchar *misc_font);
 
 void
 gnome_cmd_data_set_right_mouse_button_mode (GnomeCmdRightMouseButtonMode mode);
@@ -200,40 +235,137 @@ gnome_cmd_data_set_term                (const gchar *shell);
 const gchar *
 gnome_cmd_data_get_term                (void);
 
-gint
-gnome_cmd_data_get_embterm_height      (void);
-
-void
-gnome_cmd_data_set_embterm_height      (gint height);
-
 void
 gnome_cmd_data_set_show_toolbar        (gboolean value);
 
 gboolean
 gnome_cmd_data_get_show_toolbar        (void);
 
-gboolean
-gnome_cmd_data_get_use_mime            (void);
-
-void
-gnome_cmd_data_set_use_mime            (gboolean value);
-
 guint
-gnome_cmd_data_get_mime_icon_size      (void);
+gnome_cmd_data_get_icon_size           (void);
 
 void
-gnome_cmd_data_set_mime_icon_size      (guint size);
+gnome_cmd_data_set_icon_size           (guint size);
 
 GdkInterpType
-gnome_cmd_data_get_mime_scale_quality  (void);
+gnome_cmd_data_get_icon_scale_quality  (void);
 
 void
-gnome_cmd_data_set_mime_scale_quality  (GdkInterpType quality);
+gnome_cmd_data_set_icon_scale_quality  (GdkInterpType quality);
 
 const gchar *
-gnome_cmd_data_get_mime_icon_dir       (void); 
+gnome_cmd_data_get_theme_icon_dir      (void); 
 
 void
-gnome_cmd_data_set_mime_icon_dir       (const gchar *dir);
+gnome_cmd_data_set_theme_icon_dir      (const gchar *dir);
+
+const gchar *
+gnome_cmd_data_get_document_icon_dir   (void); 
+
+void
+gnome_cmd_data_set_document_icon_dir   (const gchar *dir);
+
+void
+gnome_cmd_data_set_fs_col_width        (guint column, gint width);
+
+gint
+gnome_cmd_data_get_fs_col_width        (guint column);
+
+void
+gnome_cmd_data_set_bookmark_dialog_col_width (guint column, gint width);
+
+gint
+gnome_cmd_data_get_bookmark_dialog_col_width (guint column);
+
+gint
+gnome_cmd_data_get_cmdline_history_length (void);
+
+void
+gnome_cmd_data_set_cmdline_history_length (gint length);
+
+GList *
+gnome_cmd_data_get_cmdline_history     (void);
+
+void
+gnome_cmd_data_set_button_relief       (GtkReliefStyle relief);
+
+GtkReliefStyle
+gnome_cmd_data_get_button_relief       (void);
+
+void
+gnome_cmd_data_set_filter_type         (FilterType type);
+
+FilterType
+gnome_cmd_data_get_filter_type         (void);
+
+void
+gnome_cmd_data_set_device_only_icon    (gboolean value);
+
+gboolean
+gnome_cmd_data_get_device_only_icon    (void);
+
+void
+gnome_cmd_data_set_dir_cache_size      (gint size);
+
+gint
+gnome_cmd_data_get_dir_cache_size      (void);
+
+void
+gnome_cmd_data_set_use_ls_colors       (gboolean value);
+
+gboolean
+gnome_cmd_data_get_use_ls_colors       (void);
+
+void
+gnome_cmd_data_set_use_fam             (gboolean value);
+
+gboolean
+gnome_cmd_data_get_use_fam             (void);
+
+SearchDefaults *
+gnome_cmd_data_get_search_defaults     (void);
+
+GnomeCmdFtpServer *
+gnome_cmd_data_get_quick_connect_server (void);
+
+GnomeCmdBookmarkGroup *
+gnome_cmd_data_get_local_bookmarks      (void);
+
+GList *
+gnome_cmd_data_get_bookmark_groups      (void);
+
+gboolean
+gnome_cmd_data_get_honor_expect_uris    (void);
+
+void
+gnome_cmd_data_set_honor_expect_uris    (gboolean value);
+
+gboolean
+gnome_cmd_data_get_skip_mounting        (void);
+
+void
+gnome_cmd_data_set_skip_mounting        (gboolean value);
+
+gboolean
+gnome_cmd_data_get_toolbar_visibility   (void);
+
+gboolean
+gnome_cmd_data_get_buttonbar_visibility (void);
+
+void
+gnome_cmd_data_set_toolbar_visibility   (gboolean value);
+
+void
+gnome_cmd_data_set_buttonbar_visibility (gboolean value);
+
+AdvrenameDefaults *
+gnome_cmd_data_get_advrename_defaults   (void);
+
+void
+gnome_cmd_data_set_last_pattern (const gchar *value);
+
+const gchar *
+gnome_cmd_data_get_last_pattern (void);
+
 
 #endif //__GNOME_CMD_DATA_H__
