@@ -47,7 +47,8 @@ struct _GnomeCmdDirPrivate
 	GList *infolist;
 	gboolean done;
 	gboolean lock;
-	gint count;	
+	gint count;
+	GnomeVFSResult last_result;
 };
 
 static GtkObjectClass *parent_class = NULL;
@@ -152,7 +153,6 @@ init (GnomeCmdDir *dir)
 	dir->files = NULL;
 	dir->content_changed = FALSE;
 	dir->prev_error = FALSE;
-	dir->not_listed = TRUE;
 	dir->voffset = 0;
 	dir_counter++;
 }
@@ -300,6 +300,16 @@ void gnome_cmd_dir_unref (GnomeCmdDir *dir)
 }
 
 
+GnomeVFSResult gnome_cmd_dir_get_files (GnomeCmdDir *dir, GList **files)
+{
+	g_return_val_if_fail (GNOME_CMD_IS_DIR (dir), GNOME_VFS_ERROR_BAD_PARAMETERS);
+
+	*files = dir->files;
+
+	return dir->priv->last_result;
+}
+
+
 /******************************************************************************
 *
 *   Function: gnome_cmd_dir_list_files
@@ -318,21 +328,18 @@ void gnome_cmd_dir_unref (GnomeCmdDir *dir)
 *
 ******************************************************************************/
 
-GList *gnome_cmd_dir_list_files (GnomeCmdDir *dir, gboolean visprog)
+void gnome_cmd_dir_list_files (GnomeCmdDir *dir, gboolean visprog)
 {
-	g_return_val_if_fail (GNOME_CMD_IS_DIR (dir), NULL);
+	g_return_if_fail (GNOME_CMD_IS_DIR (dir));
 
-	if (!dir->files || dir->content_changed || dir->not_listed)
+	if (!dir->files || dir->content_changed	|| gnome_vfs_uri_is_local (dir->uri))
 	{
-		DEBUG ('l', "going to relist in list due to content change or no previous files where present\n");
-		return gnome_cmd_dir_relist_files (dir, visprog);
+		DEBUG ('l', "relisting files\n");
+		gnome_cmd_dir_relist_files (dir, visprog);		
 	}
-	else
-	{
-		DEBUG ('l', "list\n");
+	else {
+		gtk_signal_emit (GTK_OBJECT (dir), dir_signals[LIST_UPDATED]);
 	}
-
-	return dir->files;
 }
 
 
@@ -438,6 +445,8 @@ on_files_listed (GnomeVFSAsyncHandle *handle,
 {
 	g_return_if_fail (GNOME_CMD_IS_DIR (dir));
 
+	dir->priv->last_result = result;
+	
 	if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
 		warn_print ("Directory listing failed, %s\n", gnome_vfs_result_to_string (result));	
 		dir->prev_error = TRUE;
@@ -461,6 +470,7 @@ on_files_listed (GnomeVFSAsyncHandle *handle,
 	
 	if (result == GNOME_VFS_ERROR_EOF) {
 		dir->priv->done = TRUE;
+		dir->priv->last_result = GNOME_VFS_OK;
 		return;
 	}
 }
@@ -472,6 +482,8 @@ visprog_list (GnomeCmdDir *dir)
 {
 	GnomeVFSFileInfoOptions infoOpts;
 
+	DEBUG ('l', "visprog_list\n");
+	
 	infoOpts = GNOME_VFS_FILE_INFO_FOLLOW_LINKS|GNOME_VFS_FILE_INFO_GET_MIME_TYPE;
 
 	dir->priv->done = FALSE;
@@ -501,10 +513,11 @@ visprog_list (GnomeCmdDir *dir)
 static GList *
 blocking_list (GnomeCmdDir *dir)
 {
-	GnomeVFSResult result;
 	GnomeVFSFileInfoOptions infoOpts;
 	GnomeVFSDirectoryFilter *filter;
 
+	DEBUG ('l', "blocking_list\n");
+
 	infoOpts = GNOME_VFS_FILE_INFO_FOLLOW_LINKS|GNOME_VFS_FILE_INFO_GET_MIME_TYPE;
 
 	dir->priv->done = FALSE;
@@ -516,7 +529,7 @@ blocking_list (GnomeCmdDir *dir)
 		GNOME_VFS_DIRECTORY_FILTER_NOPARENTDIR,
 		NULL);
 	
-	result = gnome_vfs_directory_list_load (
+	dir->priv->last_result = gnome_vfs_directory_list_load (
 		&dir->priv->infolist,
 		gnome_cmd_dir_get_uri_str (dir),
 		infoOpts,
@@ -524,10 +537,10 @@ blocking_list (GnomeCmdDir *dir)
 
 	gnome_vfs_directory_filter_destroy (filter);
 
-	if (result == GNOME_VFS_OK)
+	if (dir->priv->last_result == GNOME_VFS_OK)
 		return dir->priv->infolist;
 
-	create_error_dialog (gnome_vfs_result_to_string (result));
+	create_error_dialog (gnome_vfs_result_to_string (dir->priv->last_result));
 	
 	return NULL;
 }
@@ -549,21 +562,21 @@ blocking_list (GnomeCmdDir *dir)
 *
 ******************************************************************************/
 
-GList *gnome_cmd_dir_relist_files (GnomeCmdDir *dir, gboolean visprog)
+void gnome_cmd_dir_relist_files (GnomeCmdDir *dir, gboolean visprog)
 {
 	GList *list;
 	GList *tmp;
 	
-	if (dir->priv->lock) return NULL;
+	if (dir->priv->lock) return;
 	dir->priv->lock = TRUE;
 	
-	g_return_val_if_fail (GNOME_CMD_IS_DIR (dir), NULL);
-	g_return_val_if_fail (dir->uri != NULL, NULL);
+	g_return_if_fail (GNOME_CMD_IS_DIR (dir));
+	g_return_if_fail (dir->uri != NULL);
 
 	if (dir->prev_error) {
 		DEBUG ('l', "Skipping relist due to previous error\n");
 		dir->priv->lock = FALSE;
-		return NULL;
+		return;
 	}
 	
 	DEBUG ('l', "relist\n");
@@ -608,14 +621,9 @@ GList *gnome_cmd_dir_relist_files (GnomeCmdDir *dir, gboolean visprog)
 	
 	dir->content_changed = FALSE;
 	
-	/* only send the list-updated signal if this is not the first relist */
-	if (dir->not_listed)
-		dir->not_listed = FALSE;
-	else
-		gtk_signal_emit (GTK_OBJECT (dir), dir_signals[LIST_UPDATED]);
+	gtk_signal_emit (GTK_OBJECT (dir), dir_signals[LIST_UPDATED]);
 
 	dir->priv->lock = FALSE;
-	return dir->files;
 }
 
 
