@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #ifndef __GNOME_CMD_FILE_POPMENU_H__
 #define __GNOME_CMD_FILE_POPMENU_H__
 
+#include "gnome-cmd-file-list.h"
 
 BEGIN_GNOME_DECLS
 
@@ -50,7 +51,7 @@ struct _GnomeCmdFilePopmenuClass
 
 
 GtkWidget*
-gnome_cmd_file_popmenu_new (GList *files);
+gnome_cmd_file_popmenu_new (GnomeCmdFileList *fl);
 
 GtkType
 gnome_cmd_file_popmenu_get_type (void);
