@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,10 +19,25 @@
 #ifndef __GNOME_CMD_FILE_TYPES_H__
 #define __GNOME_CMD_FILE_TYPES_H__
 
-#define GNOME_CMD_FILE(obj)		   ((GnomeCmdFile*) obj)
+BEGIN_GNOME_DECLS
 
-typedef struct
+#define GNOME_CMD_FILE(obj) \
+	GTK_CHECK_CAST (obj, gnome_cmd_file_get_type (), GnomeCmdFile)
+#define GNOME_CMD_FILE_CLASS(klass) \
+	GTK_CHECK_CLASS_CAST (klass, gnome_cmd_file_get_type (), GnomeCmdFileClass)
+#define GNOME_CMD_IS_FILE(obj) \
+    GTK_CHECK_TYPE (obj, gnome_cmd_file_get_type ())
+
+
+typedef struct _GnomeCmdFile GnomeCmdFile;
+typedef struct _GnomeCmdFileClass GnomeCmdFileClass;
+
+#include "gnome-cmd-dir-types.h"
+
+struct _GnomeCmdFile
 {
+	GtkObject parent;
+	
 	GnomeVFSFileInfo *info;
 	GnomeCmdDir *dir;
 	GnomeVFSURI *uri;
@@ -30,11 +45,12 @@ typedef struct
 	gchar *path;
 	gint ref_cnt;
 	GList *observers;
-} GnomeCmdFile;
+};
 
-typedef void (* GnomeCmdFileUpdateCallbackFunc) (
-	GnomeCmdFile *file,
-	gpointer user_data);
+struct _GnomeCmdFileClass
+{
+	GtkObjectClass parent_class;
+};
 
 
 #endif //__GNOME_CMD_FILE_TYPES_H__
