@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,6 +18,10 @@
 */ 
 #include <config.h>
 #include "gnome-cmd-includes.h"
+#include "gnome-cmd-mkdir-dialog.h"
+#include "gnome-cmd-dir-funcs.h"
+#include "gnome-cmd-main-win-funcs.h"
+#include "utils.h"
 
 
 struct _GnomeCmdMkdirDialogPrivate
@@ -31,17 +35,21 @@ static GnomeCmdStringDialogClass *parent_class = NULL;
 
 
 
-static void
-on_ok (const gchar *string, GnomeCmdMkdirDialog *dialog)
+static gboolean
+on_ok (GnomeCmdStringDialog *string_dialog, const gchar **values, GnomeCmdMkdirDialog *dialog)
 {
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
-
+	const gchar *filename = values[0];
+	
 	/* dont create any directory if no name was passed or cancel was selected */
-	if (string == NULL || strcmp (string, "") == 0)
-		return;
+	if (filename == NULL || strcmp (filename, "") == 0) {
+		gnome_cmd_string_dialog_set_error_desc (string_dialog,
+												g_strdup (_("A directory name must be entered")));
+		return FALSE;
+	}
 
-	uri = gnome_cmd_dir_get_file_uri (dialog->priv->dir, string);
+	uri = gnome_cmd_dir_get_file_uri (dialog->priv->dir, filename);
 
 	result = gnome_vfs_make_directory_for_uri (
 		uri,
@@ -51,10 +59,17 @@ on_ok (const gchar *string, GnomeCmdMkdirDialog *dialog)
 
 	gnome_vfs_uri_unref (uri);
 	
-	if (result != GNOME_VFS_OK)
-		create_error_dialog (gnome_vfs_result_to_string (result));
-
-	gnome_cmd_dir_unref (dialog->priv->dir);
+	if (result == GNOME_VFS_OK) {
+		gnome_cmd_dir_file_created (dialog->priv->dir, filename);
+		gnome_cmd_file_list_focus_file (gnome_cmd_main_win_get_active_fs (main_win)->list,
+										filename, TRUE);
+		gnome_cmd_dir_unref (dialog->priv->dir);
+		return TRUE;
+	}
+	
+	gnome_cmd_string_dialog_set_error_desc (string_dialog,
+											g_strdup (gnome_vfs_result_to_string (result)));
+	return FALSE;
 }
 
 
@@ -119,18 +134,21 @@ init (GnomeCmdMkdirDialog *dialog)
 
 GtkWidget*
 gnome_cmd_mkdir_dialog_new (GnomeCmdDir *dir)
-{	
+{
+	const gchar *labels[] = {_("Directory name:"), NULL};
 	GnomeCmdMkdirDialog *dialog = gtk_type_new (gnome_cmd_mkdir_dialog_get_type ());
 
 	dialog->priv->dir = dir;
 	gnome_cmd_dir_ref (dir);
-	
-	gnome_cmd_string_dialog_set_title (GNOME_CMD_STRING_DIALOG (dialog), _("Make directory"));
-	gnome_cmd_string_dialog_set_prompt (GNOME_CMD_STRING_DIALOG (dialog), _("Enter a name for the new directory"));
-	gnome_cmd_string_dialog_set_pixmap (GNOME_CMD_STRING_DIALOG (dialog), PIXMAP_MKDIR);
-	gnome_cmd_string_dialog_set_ok_cb (GNOME_CMD_STRING_DIALOG (dialog), (GnomeStringCallback)on_ok);
-	gnome_cmd_string_dialog_set_cancel_cb (GNOME_CMD_STRING_DIALOG (dialog), (GnomeStringCallback)on_cancel);
-	gnome_cmd_string_dialog_set_userdata (GNOME_CMD_STRING_DIALOG (dialog), dialog);
+
+	gnome_cmd_string_dialog_setup_with_cancel (		
+		GNOME_CMD_STRING_DIALOG (dialog),
+		_("Make Directory"),
+		labels,
+		1,
+		(GnomeCmdStringDialogCallback)on_ok,
+		(GnomeStringCallback)on_cancel,
+		dialog);
 	
 	return GTK_WIDGET (dialog);
 }
