@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +15,16 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/ 
-extern GtkStyle *main_style,*list_style;
+*/
+
+#ifndef __GNOME_CMD_STYLE_H__
+#define __GNOME_CMD_STYLE_H__
+
+
+extern GtkRcStyle *main_rc_style;
+extern GtkStyle *list_style, *sel_list_style;
 
 void gnome_cmd_style_create (void);
+
+
+#endif //__GNOME_CMD_STYLE_H__
