@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,13 +19,16 @@
 #ifndef __GNOME_CMD_TYPES_H__
 #define __GNOME_CMD_TYPES_H__
 
+#include "gnome-cmd-ftp-server-types.h"
+
 
 typedef char* GnomeCmdDateFormat;
 
 typedef enum
 {
 	GNOME_CMD_LAYOUT_TEXT,
-	GNOME_CMD_LAYOUT_ICONS
+	GNOME_CMD_LAYOUT_TYPE_ICONS,
+	GNOME_CMD_LAYOUT_MIME_ICONS
 } GnomeCmdLayout;
 
 
@@ -37,10 +40,26 @@ typedef enum
 } GnomeCmdSizeDispMode;
 
 
-typedef gint (* GnomeCmdStartXferFunc) (GSList *src_files,
-										GnomeVFSURI *src_dir,
-										GnomeVFSURI *dest_dir,
-										GnomeVFSXferOptions xferOptions,
-										GnomeVFSXferOverwriteMode xferOverwriteMode);
+typedef enum
+{
+	GNOME_CMD_PERM_DISP_MODE_TEXT,
+	GNOME_CMD_PERM_DISP_MODE_NUMBER
+} GnomeCmdPermDispMode;
+
+
+typedef struct {
+	gchar *alias;
+	GList *bookmarks;
+	GnomeCmdFtpServer *server;
+	gpointer *data;
+} GnomeCmdBookmarkGroup;
+
+
+typedef struct {
+	gchar *name;
+	gchar *path;
+	GnomeCmdBookmarkGroup *group;
+} GnomeCmdBookmark;
+
 
 #endif //__GNOME_CMD_TYPES_H__
