@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,32 +18,19 @@
 */ 
 
 
+#ifndef __IMAGELOADER_H__
+#define __IMAGELOADER_H__
+
+#include "gnome-cmd-pixmap.h"
+
+
 /**
  * If you add a pixmap id here be sure to add its filename in 
  * the array in imageloader.c
  */
 typedef enum {
-	PIXMAP_FILE_TYPE_UNKNOWN,
-	PIXMAP_FILE_TYPE_REGULAR,
-	PIXMAP_FILE_TYPE_DIRECTORY,
-	PIXMAP_FILE_TYPE_FIFO,
-	PIXMAP_FILE_TYPE_SOCKET,
-	PIXMAP_FILE_TYPE_CHAR_DEVICE,
-	PIXMAP_FILE_TYPE_BLOCK_DEVICE,
-	PIXMAP_FILE_TYPE_SYMLINK,
+	PIXMAP_NONE,
 	
-	PIXMAP_FILE_TYPE_UNKNOWN_BIG,
-	PIXMAP_FILE_TYPE_REGULAR_BIG,
-	PIXMAP_FILE_TYPE_DIRECTORY_BIG,
-	PIXMAP_FILE_TYPE_FIFO_BIG,
-	PIXMAP_FILE_TYPE_SOCKET_BIG,
-	PIXMAP_FILE_TYPE_CHAR_DEVICE_BIG,
-	PIXMAP_FILE_TYPE_BLOCK_DEVICE_BIG,
-	PIXMAP_FILE_TYPE_SYMLINK_BIG,
-
-	PIXMAP_FILE_TYPE_EXE_BIG,
-	PIXMAP_CVS_FISH_SMALL,
-
 	PIXMAP_FLIST_ARROW_UP,
 	PIXMAP_FLIST_ARROW_DOWN,
 	PIXMAP_FLIST_ARROW_BLANK,
@@ -53,18 +40,36 @@ typedef enum {
 	PIXMAP_EXEC,
 	PIXMAP_MKDIR,
 	PIXMAP_LOCK,
+	PIXMAP_HOME,
+	PIXMAP_SMB_NETWORK,
+	PIXMAP_SMB_COMPUTER,
+	PIXMAP_BOOKMARK,
+
+	PIXMAP_OVERLAY_SYMLINK,
+	PIXMAP_OVERLAY_UMOUNT,
+	PIXMAP_PARENT_DIR,
+	PIXMAP_ROOT_DIR,
+	PIXMAP_FTP_DISCONNECT,
 	
 	NUM_PIXMAPS
 } Pixmap;
 
 
-void IMAGE_init (GtkWidget *widget);
+void IMAGE_init (void);
+void IMAGE_free (void);
 
 GdkPixmap *IMAGE_get_pixmap (Pixmap pixmap_id);
 GdkBitmap *IMAGE_get_mask (Pixmap pixmap_id);
-gboolean IMAGE_get_mime_pixmap_and_mask (const gchar *mime_type,
-										 GdkPixmap **pixmap,
-										 GdkBitmap **bitmap,
-										 gboolean big);
+GdkPixbuf *IMAGE_get_pixbuf (Pixmap pixmap_id);
+GnomeCmdPixmap *IMAGE_get_gnome_cmd_pixmap (Pixmap pixmap_id);
+
+gboolean IMAGE_get_pixmap_and_mask (GnomeVFSFileType type,
+									const gchar *mime_type,
+									gboolean symlink,
+									GdkPixmap **pixmap,
+									GdkBitmap **mask);
 
 void IMAGE_clear_mime_cache (void);
+
+
+#endif //__IMAGELOADER_H__
