@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2002 Marcus Bjurman
+    Copyright (C) 2001-2003 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,8 @@
 */ 
 #include <config.h>
 #include "gnome-cmd-includes.h"
-
+#include "owner.h"
+#include "utils.h"
 
 GList *all_users;
 GList *all_groups;
